//+------------------------------------------------------------------+
//|                                                                  |
//| "Fixing the Bollinger bands"                                     |
//| Futures magazine 05.2010, David Rooke                            }
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers    1
#property indicator_color1     Red         
#property indicator_width1     2
#property indicator_level1     0
#property indicator_level2     0.5
#property indicator_level3     1
#property indicator_levelcolor DimGray


//
//
//
//
//

extern int    Length                 = 120;//20 
extern int    Price                  = PRICE_CLOSE;
extern double Deviations             = 2.0;
extern bool   UseClassicalDeviations = false; 
extern double SmoothPeriod           = 2.5;
extern double SmoothPhase            = 0;
extern bool   ShowArrows             = true;
extern double ShowArrowsWhenValue    = 0.5;
extern string arrowsIdentifier       = "volatility bands - percent";
extern color  arrowsUpColor          = DeepSkyBlue;
extern color  arrowsDnColor          = Red;

extern bool   alertsOn          = False;
extern bool   alertsOnCurrent   = true;
extern bool   alertsMessage     = true;
extern bool   alertsSound       = false;
extern bool   alertsEmail       = false;

//
//
//
//
//

double vbp[];
double vbc[];
double vbu[];
double vbd[];
double prices[];
double trend[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0, vbp); 
   SetIndexBuffer(1, vbc); 
   SetIndexBuffer(2, vbu); 
   SetIndexBuffer(3, vbd); 
   SetIndexBuffer(4, prices); 
   SetIndexBuffer(5, trend); 
   return(0);
}
int deinit()
{
   if (ShowArrows) deleteArrows();
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         
   //
   //
   //
   //
   //

   for(i=limit; i>=0; i--)
   {
      prices[i] = iMA(NULL,0,1,0,MODE_SMA,Price,i);

         double tema1 = iTema(prices[i],Length,i,0);
         double tema2 = iTema(tema1    ,Length,i,3);

      vbc[i] = 2.0*tema1-tema2;

         double deviation = iDeviationPlus(prices,vbc,Length,i);

      vbu[i] = vbc[i]+deviation*Deviations;
      vbd[i] = vbc[i]-deviation*Deviations;

         double width = vbu[i]-vbd[i];

      if (width>0)
           vbp[i] = iSmooth((prices[i]-vbd[i])/width,SmoothPeriod,SmoothPhase,i);
      else vbp[i] = iSmooth(0                       ,SmoothPeriod,SmoothPhase,i);

      //
      //
      //
      //
      //
      
      manageArrow(i);
   }
   manageAlerts();
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (ShowArrows)
   {
      trend[i] = trend[i+1];
         if (vbp[i]>ShowArrowsWhenValue) trend[i] =  1;
         if (vbp[i]<ShowArrowsWhenValue) trend[i] = -1;
         
         deleteArrow(Time[i]);
         if (trend[i]!=trend[i+1])
         {
            if (trend[i] == 1) drawArrow(i,arrowsUpColor,241,false);
            if (trend[i] ==-1) drawArrow(i,arrowsDnColor,242,true);
         }
   }
}               

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}
void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," volatility bands trend changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"  volatility bands "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double wrk[][6];

double iTema(double price, int length, int i, int s=0)
{
   if (ArrayRange(wrk,0)!=Bars) ArrayResize(wrk,Bars);
   
   //
   //
   //
   //
   //
   
   double alpha = 2.0 /(1.0 + length);
   int    r = Bars-i-1;
   
   if (r < 1)
      {
         wrk[r][0+s] = price;
         wrk[r][1+s] = price;
         wrk[r][2+s] = price;
      }
   else
      {
         wrk[r][0+s] = wrk[r-1][0+s]+alpha*(price      -wrk[r-1][0+s]);
         wrk[r][1+s] = wrk[r-1][1+s]+alpha*(wrk[r][0+s]-wrk[r-1][1+s]);
         wrk[r][2+s] = wrk[r-1][2+s]+alpha*(wrk[r][1+s]-wrk[r-1][2+s]);
      }
   return(3*wrk[r][0+s] - 3*wrk[r][1+s] + wrk[r][2+s]);
}

//
//
//
//    In the original article the second method is how deviation is
//    calculated . It is a questionableble method , but if we apply
//    standard calculation the bands tend to be "spiky".That is the
//    reason to have the "UseClassicalDeviations" option
//
//
//

double iDeviationPlus(double& array[], double& ma[], int period, int i)
{
   double sum = 0;
      if (UseClassicalDeviations)
            for(int k=0; k<period; k++) sum += (array[i+k]-ma[i])  *(array[i+k]-ma[i]);
      else  for(    k=0; k<period; k++) sum += (array[i+k]-ma[i+k])*(array[i+k]-ma[i+k]);       
   return(MathSqrt(sum/period));
}

//+------------------------------------------------------------------
//|                                                                 
//+------------------------------------------------------------------
//
//
//
//
//

double wrkj[][10];

#define bsmax  5
#define bsmin  6
#define volty  7
#define vsum   8
#define avolty 9

//
//
//
//
//

double iSmooth(double price, double length, double phase, int i, int s=0)
{
   if (ArrayRange(wrkj,0) != Bars) ArrayResize(wrkj,Bars);
   if (length <=1) return(price);
   
   int r = Bars-i-1; 
      if (r==0) { for(int k=0; k<7; k++) wrkj[0][k+s]=price; for(; k<10; k++) wrkj[0][k+s]=0; return(price); }

   //
   //
   //
   //
   //
   
      double len1   = MathMax(MathLog(MathSqrt(0.5*(length-1)))/MathLog(2.0)+2.0,0);
      double pow1   = MathMax(len1-2.0,0.5);
      double del1   = price - wrkj[r-1][bsmax+s];
      double del2   = price - wrkj[r-1][bsmin+s];
      double div    = 1.0/(10.0+10.0*(MathMin(MathMax(length-10,0),100))/100);
      int    forBar = MathMin(r,10);
	
         wrkj[r][volty+s] = 0;
               if(MathAbs(del1) > MathAbs(del2)) wrkj[r][volty+s] = MathAbs(del1); 
               if(MathAbs(del1) < MathAbs(del2)) wrkj[r][volty+s] = MathAbs(del2); 
         wrkj[r][vsum+s] =	wrkj[r-1][vsum+s] + (wrkj[r][volty+s]-wrkj[r-forBar][volty+s])*div;
         
         //
         //
         //
         //
         //
   
         wrkj[r][avolty+s] = wrkj[r-1][avolty+s]+(2.0/(MathMax(4.0*length,30)+1.0))*(wrkj[r][vsum+s]-wrkj[r-1][avolty+s]);
            if (wrkj[r][avolty+s] > 0)
               double dVolty = wrkj[r][volty+s]/wrkj[r][avolty+s]; else dVolty = 0;   
	               if (dVolty > MathPow(len1,1.0/pow1)) dVolty = MathPow(len1,1.0/pow1);
                  if (dVolty < 1)                      dVolty = 1.0;

      //
      //
      //
      //
      //
	        
   	double pow2 = MathPow(dVolty, pow1);
      double len2 = MathSqrt(0.5*(length-1))*len1;
      double Kv   = MathPow(len2/(len2+1), MathSqrt(pow2));

         if (del1 > 0) wrkj[r][bsmax+s] = price; else wrkj[r][bsmax+s] = price - Kv*del1;
         if (del2 < 0) wrkj[r][bsmin+s] = price; else wrkj[r][bsmin+s] = price - Kv*del2;
	
   //
   //
   //
   //
   //
      
      double R     = MathMax(MathMin(phase,100),-100)/100.0 + 1.5;
      double beta  = 0.45*(length-1)/(0.45*(length-1)+2);
      double alpha = MathPow(beta,pow2);

         wrkj[r][0+s] = price + alpha*(wrkj[r-1][0+s]-price);
         wrkj[r][1+s] = (price - wrkj[r][0+s])*(1-beta) + beta*wrkj[r-1][1+s];
         wrkj[r][2+s] = (wrkj[r][0+s] + R*wrkj[r][1+s]);
         wrkj[r][3+s] = (wrkj[r][2+s] - wrkj[r-1][4+s])*MathPow((1-alpha),2) + MathPow(alpha,2)*wrkj[r-1][3+s];
         wrkj[r][4+s] = (wrkj[r-1][4+s] + wrkj[r][3+s]); 

   //
   //
   //
   //
   //

   return(wrkj[r][4+s]);
}